@@ -5,12 +5,15 @@ use strict;
 use Params::Validate qw( validate SCALAR );
 
 use overload ( fallback => 1,
-               'cmp' => 'compare',
-               '<=>' => 'compare',
+               'cmp' => '_compare_overload',
+               '<=>' => '_compare_overload',
                '+'   => '_add_overload',
                '-'   => '_subtract_overload',
+               '*'   => '_multiply_overload',
              );
 
+use constant MAX_NANOSECONDS => 1000000000;  # 1E9 = almost 32 bits
+
 sub new
 {
     my $class = shift;
@@ -22,6 +25,7 @@ sub new
                            hours   => { type => SCALAR, default => 0 },
                            minutes => { type => SCALAR, default => 0 },
                            seconds => { type => SCALAR, default => 0 },
+                           nanoseconds => { type => SCALAR, default => 0 },
                            end_of_month => { type => SCALAR, default => 'wrap',
                                              regex => qr/^(?:wrap|limit|preserve)$/ },
                          } );
@@ -39,32 +43,72 @@ sub new
         $self->{sign} = 1;
     }
 
-    $self->{months} = ( abs( $p{years} * 12 ) + abs( $p{months} ) ) * $self->{sign};
-    $self->{days}   = ( abs( ( $p{weeks} * 7 ) ) + abs( $p{days} ) ) * $self->{sign};
-    $self->{seconds} =
-        abs( ( $p{hours} * 3600 ) + ( $p{minutes} * 60 ) + $p{seconds} ) * $self->{sign};
+    $self->{months} =
+        ( abs( $p{years} * 12 ) + abs( $p{months} ) ) * $self->{sign};
+
+    $self->{days} =
+        ( abs( ( $p{weeks} * 7 ) ) + abs( $p{days} ) ) * $self->{sign};
+
+    $self->{minutes} = abs( ( $p{hours} * 60 ) + $p{minutes}  ) * $self->{sign};
+
+    $self->{seconds} = abs( $p{seconds} ) * $self->{sign};
+
+    if ( $p{nanoseconds} )
+    {
+        $self->{nanoseconds} = abs( $p{nanoseconds} ) * $self->{sign};
+        $self->_normalize_nanoseconds;
+    }
+    else
+    {
+        # shortcut - if they don't need nanoseconds
+        $self->{nanoseconds} = 0;
+    }
 
     return $self;
 }
 
+sub _normalize_nanoseconds
+{
+    my $self = shift;
+
+    if ( $self->{nanoseconds} < 0 )
+    {
+        my $overflow = int( $self->{nanoseconds} / MAX_NANOSECONDS );
+        $self->{nanoseconds} += $overflow * MAX_NANOSECONDS;
+        $self->{seconds} -= $overflow;
+    }
+    elsif ( $self->{nanoseconds} >= MAX_NANOSECONDS )
+    {
+        my $overflow = int( $self->{nanoseconds} / MAX_NANOSECONDS );
+        $self->{nanoseconds} -= $overflow * MAX_NANOSECONDS;
+        $self->{seconds} += $overflow;
+    }
+}
+
 sub clone { bless { %{ $_[0] } }, ref $_[0] }
 
 sub years   { abs( int( $_[0]->{months} / 12 ) ) }
 sub months  { int( abs( $_[0]->{months} ) % 12 ) }
 sub weeks   { abs( int( $_[0]->{days} / 7 ) ) }
 sub days    { abs( $_[0]->{days} ) % 7 }
-sub hours   { abs( int( $_[0]->{seconds} / 3600 ) ) }
-sub minutes { int( ( abs( $_[0]->{seconds} ) - ( $_[0]->hours * 3600 ) ) / 60 ) }
-sub seconds { abs( $_[0]->{seconds} ) % 60 }
+sub hours   { abs( int( $_[0]->{minutes} / 60 ) ) }
+sub minutes { int( ( abs( $_[0]->{minutes} ) - ( $_[0]->hours * 60 ) ) ) }
+sub seconds { abs( $_[0]->{seconds} ) }
+sub nanoseconds { abs( $_[0]->{nanoseconds} ) }
 
 sub is_positive { $_[0]->{sign} == 1 ? 1 : 0 }
 sub is_negative { $_[0]->{sign} == -1 ? 1 : 0 }
 
 sub delta_months  { $_[0]->{months} }
 sub delta_days    { $_[0]->{days} }
+sub delta_minutes { $_[0]->{minutes} }
 sub delta_seconds { $_[0]->{seconds} }
+sub delta_nanoseconds { $_[0]->{nanoseconds} }
 
-sub deltas { map { $_ => $_[0]->{$_} } qw( months days seconds ) }
+sub deltas
+{
+    map { $_ => $_[0]->{$_} } qw( months days minutes seconds nanoseconds );
+}
 
 sub is_wrap_mode     { $_[0]->{eom_mode} eq 'wrap'   ? 1 : 0 }
 sub is_limit_mode    { $_[0]->{eom_mode} eq 'limit'  ? 1 : 0 }
@@ -76,7 +120,7 @@ sub inverse
 
     my %new = %$self;
 
-    foreach ( qw( months days seconds sign ) )
+    foreach ( qw( months days minutes seconds nanoseconds sign ) )
     {
         $new{$_} *= -1;
     }
@@ -87,18 +131,28 @@ sub add_duration
 {
     my ( $self, $dur ) = @_;
 
-    foreach ( qw( months days seconds ) )
+    foreach ( qw( months days minutes seconds nanoseconds ) )
     {
         $self->{$_} += $dur->{$_};
     }
 
+    # we might have to normalize_nanoseconds before comparing durations
+    $self->_normalize_nanoseconds if $self->{nanoseconds};
 }
 
-sub add { shift->add_duration( (ref $_[0])->new(@_) ) }
+sub add
+{
+    my $self = shift;
+    $self->add_duration( (ref $self)->new(@_) )
+}
 
 sub subtract_duration { $_[0]->add_duration( $_[1]->inverse ) }
 
-sub subtract { shift->subtract_duration( (ref $_[0])->new(@_) ) }
+sub subtract
+{
+    my $self = shift;
+    $self->subtract_duration( (ref $self)->new(@_) )
+}
 
 sub _add_overload
 {
@@ -132,6 +186,20 @@ sub _subtract_overload
     return $new;
 }
 
+sub _multiply_overload
+{
+    my ( $self, $times ) = @_;
+
+    my $new = $self->clone;
+
+    foreach ( qw( months days minutes seconds nanoseconds ) )
+    {
+        $new->{$_} *= $times;
+    }
+
+    return $new;
+}
+
 sub compare
 {
     my ( $class, $dur1, $dur2 ) = ref $_[0] ? ( undef, @_ ) : @_;
@@ -139,7 +207,7 @@ sub compare
     my %deltas1 = $dur1->deltas;
     my %deltas2 = $dur2->deltas;
 
-    foreach ( qw( months days seconds ) )
+    foreach ( qw( months days minutes seconds nanoseconds ) )
     {
         if ( $deltas1{$_} < $deltas2{$_} )
         {
@@ -155,6 +223,16 @@ sub compare
 }
 
 
+sub _compare_overload
+{
+    my ( $d1, $d2, $rev ) = @_;
+
+    ($d1, $d2) = ($d2, $d1) if $rev;
+
+    return $d1->compare($d2);
+}
+
+
 1;
 
 __END__
@@ -173,7 +251,8 @@ DateTime::Duration - Duration objects for date math
                                 days    => 1,
                                 hours   => 6,
                                 minutes => 15,
-                                seconds => 45, );
+                                seconds => 45, 
+                                nanoseconds => 12000 );
 
   # Human-readable accessors, always positive
   $d->years;
@@ -183,6 +262,7 @@ DateTime::Duration - Duration objects for date math
   $d->hours;
   $d->minutes;
   $d->seconds;
+  $d->nanoseconds;
   $d->sign;
 
   if ( $d->is_positive ) { ... }
@@ -191,7 +271,9 @@ DateTime::Duration - Duration objects for date math
   # The important parts for date math
   $d->delta_months
   $d->delta_days
+  $d->delta_minutes
   $d->delta_seconds
+  $d->delta_nanoseconds
 
   my %deltas = $d->deltas
 
@@ -222,8 +304,8 @@ DateTime::Duration has the following methods:
 =item * new( ... )
 
 This method takes the parameters "years", "months", "weeks", "days",
-"hours", "minutes", "seconds", and "end_of_month".  All of these
-except "end_of_month" are numbers.  If any of the numbers are
+"hours", "minutes", "seconds", "nanoseconds", and "end_of_month".  All
+of these except "end_of_month" are numbers.  If any of the numbers are
 negative, the entire duration is negative.
 
 Internally, years as just treated as 12 months.  Similarly, weeks are
@@ -254,7 +336,7 @@ month to Feb 29, 2000 will result in Mar 31, 2000.
 Returns a new object with the same properties as the object on which
 this method was called.
 
-=item * years, months, weeks, days, hours, minutes, seconds
+=item * years, months, weeks, days, hours, minutes, seconds, nanoseconds
 
 These methods return numbers indicating how many of the given unit the
 object representations.  These numbers are always positive.
@@ -267,7 +349,7 @@ given to the constructor.  For example:
   print $dur->years;  # prints 1
   print $dur->months; # prints 3
 
-=item * delta_months, delta_days, delta_seconds
+=item * delta_months, delta_days, delta_minutes, delta_seconds, delta_nanoseconds
 
 These methods provide the same information as those above, but in a
 way suitable for doing date math.  The numbers returned may be
@@ -275,8 +357,9 @@ positive or negative.
 
 =item * deltas
 
-Returns a hash with the keys "months", "days", and "seconds",
-containing all the delta information for the object.
+Returns a hash with the keys "months", "days", "minutes", "seconds",
+and "nanoseconds", containing all the delta information for the
+object.
 
 =item * is_positive, is_negative
 
@@ -300,8 +383,8 @@ C<add_duration()> or C<subtract_duration()>, as appropriate.
 
 =head2 Overloading
 
-Addition, subtraction, and comparison are overloaded for objects of
-this class.
+This class overload addition, subtraction, mutiplication and
+comparison.
 
 =head1 SUPPORT
 
